Screen {
    background: black;
}

Button:focus {
    text-style: bold;
}

DataTable {
    margin: 1;
    width: 100%;
    height: 3fr;
    scrollbar-color: #652a25;

}

DataTable > .datatable--header {
    text-style: bold;
    background: #652a25;
}

DataTable > .datatable--cursor {
    background: #591f19;
}


Header {
    background: rgb(44, 43, 43);
    color: #ef6456;
}

Header.-tall {
    height: 1;
}


Input {
    background: #612823;
    color: white;
    border: vkey #652a25;
    width: 100%;
    height: auto;
    margin-top: 1;
}

Label {
    margin: 1;
}

LoadingIndicator {
    height: 1;
    margin: 0;
    color:#ef6456;
    align: left middle;
    visibility: hidden;

}


TextLog {
    margin: 1;
    scrollbar-color: #652a25;
}

#text_source {
    width: 100%;
    height: 1fr;
    margin: 1;
    border: none #652a25;
    background: #090100;
}

#dt_db {
    height: 5fr;
}

#left_vert {
    width: 40%;
}


#btn_update_db {
    height: 1;
    background: #ef6456;
    content-align: center top !important;
    border: none;
}

#btn_copy {
    margin: 1 1 1 5;
}

#update_container {
    margin: 1;
    height: 1;
}